# -*- coding: utf-8 -*-

# <api>

#%% imports
#cv2, etc

import os
import sys

import numpy as np
from matplotlib import pyplot as plt
import random

import cv2
from skimage import io
from sklearn.feature_extraction import image
from skimage.util.shape import view_as_blocks
import itertools


import os
import math
import json

import numpy as np
import pandas as pd
from matplotlib import pyplot as plt
from collections import namedtuple, OrderedDict
from matplotlib import pyplot as plt

import cv2
#from facemorpher import locator # we should get rid of this in the future and interface a STASM binary directly via system call
from skimage import io
from sklearn.feature_extraction import image
from skimage.util.shape import view_as_blocks
import itertools

#%%detection
def get_window(img='./img/fre99m36_frame_1.jpg', factor=1, size='large', backend='skimage'):
    if backend=='skimage':
        im = io.imread(img)
    else:
        im = cv2.imread(img)
        
    y_offset = 50
    x, y = 504/2, 768/2-y_offset
    
    if size == 'large':
        ax1, ax2 = 280*factor, 375*factor
    elif size == 'medium':
        ax1, ax2 = 275*factor, 370*factor
    elif size == 'small':
        ax1, ax2 = 250*factor, 360*factor
    else:
        raise ValueError("size " + str(size) + " not recognized")
    
    el = cv2.ellipse2Poly((x,y), (int(ax1)/2, int(ax2)/2), 0,0,360,4)
    
    return im, el

def paint_ellipse(im, el, color=(10,200,200), thickness=2):
    
    if cv2.__version__[0] == '3': # taking care of changed cv-API
        outcol = cv2.COLOR_BGR2RGB
        lt = cv2.LINE_AA
    else:
        outcol = cv2.cv.CV_BGR2RGB
        lt = cv2.CV_AA

    cv2.polylines(im, np.int32([el]), True, color, thickness=thickness, lineType=lt)
        
    im = cv2.cvtColor(im, outcol)
    
    return im


#%% scrambling
def create_block_coords(img, block_size, jitter=0):
    """ generates a grid of block coordinates with the same dimension as the diven image
    """
    im_shape = img.shape
    ticks_x = np.arange(0, im_shape[1], step=block_size[1])[0:-1] # don't go beyond image boundaries
    ticks_y = np.arange(0, im_shape[0], step=block_size[0])[0:-1]
    ticks = (ticks_y, ticks_x)
    return ticks


def find_block_coords_inside_bak(shape, ticks):
    """ steps through the given grid and checks each block whether it's inside the given shape
    relies heavily on cv2.pointPolygonTest
    """
    ticks_y = ticks[0]
    ticks_x = ticks[1]
    bl_x = ticks_x[1] - ticks_x[0]
    bl_y = ticks_y[1] - ticks_y[0]
    
    y_insiders = []
    
    ticks_inside = []
    ids_inside = []
    idx_x = 0
    for tick_x in ticks_x:
        idx_y = 0
        for tick_y in ticks_y:
            point_tl = (tick_y, tick_x) # top left corner of a block
            tl_inside = cv2.pointPolygonTest(shape,point_tl,False) >= 0   
            if tl_inside: # the top left corner of the block is inside - what about the rest?
                y_insiders.append(tick_y)
                point_tr = (tick_y, tick_x+bl_x)
                point_br = (tick_y+bl_y, tick_x+bl_x)
                point_bl = (tick_y+bl_y, tick_x)
                tr_inside = cv2.pointPolygonTest(shape,point_tr,False) >= 0
                br_inside = cv2.pointPolygonTest(shape,point_br,False) >= 0
                bl_inside = cv2.pointPolygonTest(shape,point_bl,False) >= 0
                if tr_inside and not br_inside and not bl_inside:
                    print point_tl
                if tr_inside and br_inside and bl_inside:
                    ticks_inside.append(point_tl)
                    ids_inside.append((idx_x, idx_y)) # (idx_y, idx_x)? (idn_y, idn_x below)
                else:
                    if tick_y > 450:
                        print tick_x
                        #import pdb; pdb.set_trace()
            idx_y += 1
        idx_x += 1
    return ticks_inside, ids_inside



def find_block_coords_inside(shape, ticks):
    """ steps through the given grid and checks each block whether it's inside the given shape
    relies heavily on cv2.pointPolygonTest
    """
    ticks_x = ticks[0]
    ticks_y = ticks[1]
    bl_x = ticks_x[1] - ticks_x[0]
    bl_y = ticks_y[1] - ticks_y[0]
    
    y_insiders = []
    
    ticks_inside = []
    ids_inside = []
    idx_x = 0
    for tick_x in ticks_x:
        idx_y = 0
        for tick_y in ticks_y:
            point_tl = (tick_y, tick_x) # top left corner of a block
            tl_inside = cv2.pointPolygonTest(shape,point_tl,False) >= 0   
            if tl_inside: # the top left corner of the block is inside - what about the rest?
                y_insiders.append(tick_y)
                point_tr = (tick_y, tick_x+bl_x)
                point_br = (tick_y+bl_y, tick_x+bl_x)
                point_bl = (tick_y+bl_y, tick_x)
                tr_inside = cv2.pointPolygonTest(shape,point_tr,False) >= 0
                br_inside = cv2.pointPolygonTest(shape,point_br,False) >= 0
                bl_inside = cv2.pointPolygonTest(shape,point_bl,False) >= 0
                if tr_inside and br_inside and bl_inside:
                    ticks_inside.append(point_tl)
                    ids_inside.append((idx_x, idx_y)) # (idx_y, idx_x)? (idn_y, idn_x below)
            idx_y += 1
        idx_x += 1
    return ticks_inside, ids_inside

""" example:
img = './img/fre99m03_frame_1.jpg'
args = fd.get_stasm_args(img, plot_el=True)
#im, el = fd.get_stasm_el(img = img, **args)

ticks_inside, ids_inside = find_block_coords_inside(el, (ticks_y, ticks_x))
for point in ticks_inside:
    pt_op = (point[0]+block_size[0], point[1]+block_size[1])
    cv2.rectangle(im_demo, point,pt_op, color=(55,100,55), thickness=2)
cv2.polylines(im_demo, np.int32([el]), True, (100,55,55), thickness=2)
fd.simple_implot(im_demo)
"""

def _split_image_blocks(im, block_size):
    """ takes an image and splits it into blocks of the given size. Returns a 2D array of those blocks
    """
    # have to crop the image so we can split it into blocks:
    height = int(im.shape[0]/block_size[0]) * block_size[0]
    width = int(im.shape[1]/block_size[1]) * block_size[1]
    im = im[0:height, 0:width]
        
    blocks = []
    blocks = map(lambda x : np.split(x, im.shape[1]/block_size[1], 1), # Split the columns
                            np.split(im, im.shape[0]/block_size[0], 0)) # Split the rows
    #import pdb
    #pdb.set_trace()
    blocks = np.array(blocks)
    return blocks

def apply_block_list(blocks, ids_inside, shuffle=False):
    """
    takes in a list of blocks and the ids of those that should be shuffled
    returns all blocks
    """
    blocks_copy = blocks.copy()
    if shuffle:
        randomized_block_list = np.random.permutation(ids_inside)
    else:
        randomized_block_list = np.array(ids_inside)
    idx_y = 0
    for row in blocks:
        idx_x = 0
        for block in row:
            this_block_id = (idx_y, idx_x)
            if this_block_id in ids_inside:
                idn_y, idn_x = randomized_block_list[0] # get random block_id that will be moved to this_block    
                randomized_block_list = np.delete(randomized_block_list,[0], axis=0)
                blocks[idx_y, idx_x] = blocks_copy[idn_y, idn_x]
            idx_x += 1
        idx_y += 1
    return blocks

def _im_from_blocks(blocks):
    """ stick list of blocks together
    """
    im = []
    im = np.hstack(blocks[0])
    import pdb
    #pdb.set_trace()
    i = 0
    for row in blocks[1:]:
        row = np.hstack(row)
        if i == 7:
            #pdb.set_trace()
            pass
        try:
            im = np.vstack([im,row])
        except:
            pass
        i +=1
    return im

# %% high level scramble funcs

# <api>
def scramble_face(im, shape, block_size=(16,16), draw_el=False, draw_grid=False, apply_list=None):
    """ main stimulus generation function
    """
    bl_x = block_size[1] # in python image processing world, it's (height/y-coord,width/x-coordinate)
    bl_y = block_size[0]
    ticks = create_block_coords(im.shape, block_size)
    ticks_y, ticks_x = ticks[0], ticks[1]
    
    if not apply_list:
        ticks_inside, ids_inside = find_block_coords_inside(shape, ticks)
        random.shuffle(ids_inside)
    else:
        ids_inside = apply_list
        ticks_inside = [(ticks[0][x[0]], ticks[1][x[1]]) for x in ids_inside]
        import pdb
        #pdb.set_trace()

    # divide actual image into blocks
    blocks = _split_image_blocks(im, block_size)
   
    #scramble
    blocks = apply_block_list(blocks, ids_inside)

    # stitch blocks together again:
    im_sc = _im_from_blocks(blocks)
   
    
    if draw_el:
        cv2.polylines(im_sc, np.int32([shape]), True, (100,55,55), thickness=1)
    
    if draw_grid:
        ticks = list(itertools.product(ticks_x, ticks_y))
        i = 0
        for point in ticks:
            pt_op = (point[0]+bl_y, point[1]+bl_x)
            if point in ticks_inside:
                color=(55,100,55)
                cv2.rectangle(im_sc, point,pt_op, color, thickness=1)
            else:
                color=(55,55,100)
                if i%2==0 or bl_x>=18:
                    cv2.rectangle(im_sc, point,pt_op, color, thickness=1)
            i += 1
        
    return im_sc, ids_inside

def scramble_directory(input_dir, output_dir, block_size=(16,16)):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    frames = os.listdir(input_dir)
    i = 0
    for frame in frames:
        source = os.path.join(input_dir, frame)
        target = os.path.join(output_dir, frame)
        args = fd.get_stasm_args(idx=source)
        (im, el) = fd.get_stasm_el(img=source, **args)
        if i==0:
            im_sc, ids_inside = scramble_face(im, el, apply_list=None, block_size=block_size)
        else:
            im_sc, ids_inside = scramble_face(im, el, apply_list=ids_inside, block_size=block_size)
        #print "saving", source, "to", target
        io.imsave(target, im_sc)
        i += 1
        
        
"""example
frame_dir = './all_frames/'
frame_scrambled_dir = './all_frames_scrambled'
stimuli = os.listdir(frame_dir)

for stim in stimuli:
    print "scrambling", stim
    source_dir = os.path.join(frame_dir, stim)
    target_dir = os.path.join(frame_scrambled_dir, stim)
    scramble_directory(source_dir, target_dir)
"""


from __future__ import division
from __future__ import unicode_literals
#%%
# -*- coding: utf-8 -*-
#%% imports
from past.utils import old_div
import cv2
import numpy as np
import random

def get_ellipse(img, shift_up=50, shift_right=5, ax_width=280, ax_height=375):
    center_x, center_y = old_div(img.shape[1],2), old_div(img.shape[0],2)
    center_x += shift_right
    center_y -= shift_up
    el = cv2.ellipse2Poly((center_x,center_y), (old_div(int(ax_width),2), old_div(int(ax_height),2)), 0,0,360,4)
    return el
#el = get_ellipse(img)

def scramble(img, el, block_size=(12,12)):
    img = img.copy()
    block_coords = create_block_coords(img, block_size)
    ticks_inside, ids_inside = find_block_coords_inside(el, block_coords)
    random.shuffle(ids_inside)    
    blocks = _split_image_blocks(img, block_size)
    blocks = apply_block_list(blocks, ids_inside)
    im_sc = _im_from_blocks(blocks)
    return im_sc

def preview_img(img, title):
    cv2.startWindowThread()
    cv2.namedWindow(title)
    cv2.imshow(title, img)    
    

#%% functions for scrambling - need to go somewhere else:
def create_block_coords(img, block_size, jitter=0):
    """ generates a grid of block coordinates with the same dimension as the diven image
    """
    im_shape = img.shape
    ticks_x = np.arange(0, im_shape[1], step=block_size[1])[0:-1] # don't go beyond image boundaries
    ticks_y = np.arange(0, im_shape[0], step=block_size[0])[0:-1]
    ticks = (ticks_y, ticks_x)
    return ticks
    
    
def find_block_coords_inside(shape, ticks):
    """ steps through the given grid and checks each block whether it's inside the given shape
    relies heavily on cv2.pointPolygonTest
    """
    ticks_x = ticks[0]
    ticks_y = ticks[1]
    bl_x = ticks_x[1] - ticks_x[0]
    bl_y = ticks_y[1] - ticks_y[0]
    
    y_insiders = []
    
    ticks_inside = []
    ids_inside = []
    idx_x = 0
    for tick_x in ticks_x:
        idx_y = 0
        for tick_y in ticks_y:
            point_tl = (tick_y, tick_x) # top left corner of a block
            tl_inside = cv2.pointPolygonTest(shape,point_tl,False) >= 0   
            if tl_inside: # the top left corner of the block is inside - what about the rest?
                y_insiders.append(tick_y)
                point_tr = (tick_y, tick_x+bl_x)
                point_br = (tick_y+bl_y, tick_x+bl_x)
                point_bl = (tick_y+bl_y, tick_x)
                tr_inside = cv2.pointPolygonTest(shape,point_tr,False) >= 0
                br_inside = cv2.pointPolygonTest(shape,point_br,False) >= 0
                bl_inside = cv2.pointPolygonTest(shape,point_bl,False) >= 0
                if tr_inside and br_inside and bl_inside:
                    ticks_inside.append(point_tl)
                    ids_inside.append((idx_x, idx_y)) # (idx_y, idx_x)? (idn_y, idn_x below)
            idx_y += 1
        idx_x += 1
    return ticks_inside, ids_inside


def _split_image_blocks(im, block_size):
    """ takes an image and splits it into blocks of the given size. Returns a 2D array of those blocks
    """
    # have to crop the image so we can split it into blocks:
    height = int(old_div(im.shape[0],block_size[0])) * block_size[0]
    width = int(old_div(im.shape[1],block_size[1])) * block_size[1]
    im = im[0:height, 0:width]
        
    blocks = []
    blocks = [np.split(x, old_div(im.shape[1],block_size[1]), 1) for x in np.split(im, old_div(im.shape[0],block_size[0]), 0)] # Split the rows

    blocks = np.array(blocks)
    return blocks

def apply_block_list(blocks, ids_inside, shuffle=False):
    """
    takes in a list of blocks and the ids of those that should be shuffled
    returns all blocks
    """
    blocks_copy = blocks.copy()
    if shuffle:
        randomized_block_list = np.random.permutation(ids_inside)
    else:
        randomized_block_list = np.array(ids_inside)
    idx_y = 0
    for row in blocks:
        idx_x = 0
        for block in row:
            this_block_id = (idx_y, idx_x)
            if this_block_id in ids_inside:
                idn_y, idn_x = randomized_block_list[0] # get random block_id that will be moved to this_block    
                randomized_block_list = np.delete(randomized_block_list,[0], axis=0)
                blocks[idx_y, idx_x] = blocks_copy[idn_y, idn_x]
            idx_x += 1
        idx_y += 1
    return blocks

def _im_from_blocks(blocks):
    """ stick list of blocks together
    """
    im = []
    im = np.hstack(blocks[0])
    i = 0
    for row in blocks[1:]:
        row = np.hstack(row)
        if i == 7:
            #pdb.set_trace()
            pass
        try:
            im = np.vstack([im,row])
        except:
            pass
        i +=1
    return im


# -*- coding: utf-8 -*-

#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

import Tkinter

class simpleapp_tk(Tkinter.Tk):
    def __init__(self,parent):
        Tkinter.Tk.__init__(self,parent)
        self.parent = parent
        self.initialize()

    def initialize(self):
        self.grid()

        self.entry = Tkinter.Entry(self)
        self.entry.grid(column=0,row=0,sticky='EW')

        button = Tkinter.Button(self,text=u"Click me !")
        button.grid(column=1,row=0)

        label = Tkinter.Label(self,
                              anchor="w",fg="white",bg="blue")
        label.grid(column=0,row=1,columnspan=2,sticky='EW')

        self.grid_columnconfigure(0,weight=1)
        self.resizable(True,False)

if __name__ == "__main__":
    app = simpleapp_tk(None)
    app.title('my application')
    app.mainloop()


from __future__ import print_function
from __future__ import unicode_literals

#%%

from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object
import tkinter as tk
from tkinter import Tk, Label, Button, StringVar, DISABLED, W
from tkinter import filedialog as tkFileDialog

import os
from face_scrambler import scrambler
from face_scrambler import helpers
import cv2


# write_cfg: helpers.dict_to_config(cfg, Config, True)

class Page(tk.LabelFrame):
    def __init__(self, parent, *args, **kwargs):
        tk.LabelFrame.__init__(self, borderwidth=5,relief=tk.GROOVE,*args, **kwargs)
        self.parent = parent
    def show(self):
        self.lift()
        
class PageLoad(Page):
    def __init__(self, parent, *args, **kwargs):
        Page.__init__(self, parent, text="1. Load an Image", *args, **kwargs)
        self.file_opt = {}
        self.file_opt['defaultextension'] = '.jpg'
        self.file_opt['filetypes'] = [('Image Files', '.jpg;.png;.png;.bmp;.jpeg'), ('All Files', '.*')]
        self.file_opt['title'] = 'Select an Image File'
        self.opn_btn_text = StringVar()
        self.opn_btn_text.set('Open File')
        load_button = Button(self, textvariable=self.opn_btn_text, command=self.openfile, pady=5)
        load_button.pack(pady=5)
        self.label_text = StringVar()
        self.label_text.set('')
        self.label_fname = Label(self, textvariable=self.label_text, pady=5)#,font="-weight bold")

    def show_buttons(self):  
        self.parent.p3.prev_button.config(state='normal')
        self.parent.p3.scramble_button.config(state='normal')
        self.parent.p3.save_button.config(state='normal')
        
    def openfile(self):
        if self.parent.mode=='dir':
            self.opendir()
            return
        cfg =helpers.config_to_dict(self.parent.Config, 'main')
        self.file_opt['initialdir'] = cfg['last_in_dir']
        self.parent.input_file = tkFileDialog.askopenfilename(**self.file_opt)
        #self.show_button.config(state='normal')
        print(self.parent.input_file)
        if self.parent.input_file:
            self.parent.img = cv2.imread(self.parent.input_file)
            self.parent.img_orig = self.parent.img.copy()
            self.parent.dirmode = False
            dir_name = os.path.dirname(self.parent.input_file)
            cfg['last_in_dir'] = dir_name
            helpers.dict_to_config(cfg, self.parent.Config, True)
            lbl_text = "Selected File: " + self.parent.input_file
            lbl_text += "\nDimensions (Height, Width, Colors): " + str(self.parent.img.shape)
            self.label_text.set(lbl_text)
            self.label_fname.pack(padx=5)
            self.show_buttons()
        
    def opendir(self):
        self.parent.all_files = []
        cfg = helpers.config_to_dict(self.parent.Config, 'main')
        self.parent.input_dir = tkFileDialog.askdirectory(title="Select a Directory Containing Images", initialdir=cfg['last_in_dir'])
        self.parent.dirmode = True
        cfg['last_in_dir'] = self.parent.input_dir
        helpers.dict_to_config(cfg, self.parent.Config, True)
        valid_images = (".jpg",".gif",".png",".tga")
        for f in os.listdir(self.parent.input_dir):
            if f.lower().endswith(valid_images):
                this_f = os.path.join(self.parent.input_dir, f)
                self.parent.all_files.append(this_f)
        n_img = len(self.parent.all_files)
        lbl_text = "Selected Directory: " + self.parent.input_dir
        #lbl_text += "\nDimensions (Height, Width, Colors): " + str(self.parent.img.shape)
        lbl_text += "\nNumber of Image Files in Directory: " + str(n_img) 
        self.label_text.set(lbl_text)
        self.label_fname.pack(padx=5)
        print(self.parent.input_dir)
        #
        self.show_buttons()
        
class PageOpt(Page):
    def __init__(self, parent, *args, **kwargs):
        Page.__init__(self, parent, text="2. Set Configuration (unit: px)", *args, **kwargs)
        cfg = parent.cfg        
        r = 0
        entryLabel = Label(self)
        entryLabel["text"] = "Size of Blocks in Pixel"
        entryLabel.grid(row=r,sticky=W)
        
        self.inp_size = tk.Spinbox(self, from_=-100, to=100, width=10)
        self.inp_size.delete(0,"end")
        self.inp_size.insert(0,int(cfg['bl_sz']))
        self.inp_size.grid(row=r, column=1, sticky=W, pady=10)
        r+=1
        
        Label(self, text='Properties of Ellipse/Mask:').grid(row=r)
        r+=1
        
        Label(self, text='Vertical Size:').grid(row=r, sticky=W, pady=5)
        self.inp_vert_size = tk.Spinbox(self, from_=-600, to=600, width=10)
        self.inp_vert_size.delete(0,"end")
        self.inp_vert_size.insert(0,int(cfg['vert_size']))
        self.inp_vert_size.grid(row=r, column=1, sticky=W, pady=5)
        r+=1
        
        Label(self, text='Horizontal Size:').grid(row=r, sticky=W, pady=5)
        self.inp_horz_size = tk.Spinbox(self, from_=-600, to=600, width=10)
        self.inp_horz_size.delete(0,"end")
        self.inp_horz_size.insert(0,int(cfg['horz_size']))
        self.inp_horz_size.grid(row=r, column=1, sticky=W, pady=5)
        r+=1
        
        entryLabel = Label(self)
        entryLabel["text"] = "Vertical Offset rel. to center\n(positive -> shift up):"
        entryLabel.grid(row=r,sticky=W)
        
        self.inp_offset_y = tk.Spinbox(self, from_=-600, to=600, width=10)
        self.inp_offset_y.delete(0,"end")
        self.inp_offset_y.insert(0,int(cfg['shift_up']))
        self.inp_offset_y.grid(row=r, column=1, sticky=W, pady=5)
        r+=1
        
        entryLabel = Label(self)
        entryLabel["text"] = "Horizontal Offset rel. to center\n(positive -> shift right):"
        entryLabel.grid(row=r,sticky=W)
        
        self.inp_offset_x = tk.Spinbox(self, from_=-600, to=600, width=10)
        self.inp_offset_x.delete(0,"end")
        self.inp_offset_x.insert(0,int(cfg['shift_right']))
        self.inp_offset_x.grid(row=r, column=1, sticky=W, pady=5)
        r+=1
        
    def save(self):
        cfg = {}
        cfg['shift_up']  = self.inp_offset_y.get()
        cfg['shift_right'] = self.inp_offset_x.get()
        cfg['horz_size']  = self.inp_horz_size.get()
        cfg['vert_size']  = self.inp_vert_size.get()
        cfg['bl_sz'] = self.inp_size.get()
        helpers.dict_to_config(cfg, self.parent.Config, True)
        
    

class PageScr(Page):
    def __init__(self, parent, *args, **kwargs):
        Page.__init__(self, parent, text="3. Scramble and Save", *args, **kwargs)
        
        self.scramble = self.parent.scramble
        r = 0
        self.label_text = StringVar()
        self.label_text.set('Selected File/Directory:')
        self.label_fname = Label(self, textvariable=self.label_text)
        #self.label_fname.grid(row=r, sticky=W, columnspan=2)
        self.label_fname.pack()
        r+=1
        
        self.scramble_button = Button(self, text="(Re-)Scramble", command=self.scramble, state=DISABLED)
        #self.scramble_button.grid(row=r, column=0)
        self.scramble_button.pack()
        r+=1
        
        self.prev_button = Button(self, text="Preview", command=self.preview_scrambled, state=DISABLED)
        #self.prev_button.grid(row=r, column=0)
        self.prev_button.pack()
        r+=1
        self.sv_btn_text = StringVar(value="Save to File")
        self.save_button = Button(self, textvariable=self.sv_btn_text, command=self.save_manual, state=DISABLED)
        self.save_button.pack()
        r+=1
        
        
    def select_output_directory(self):
        cfg = helpers.config_to_dict(self.parent.Config, 'main')
        try:
            init_dir = cfg['last_out_dir']
        except:
            init_dir = ''
        out_dir = tkFileDialog.askdirectory(title="Select a Directory Containing Images", initialdir=init_dir)
        cfg['last_out_dir'] = out_dir
        helpers.dict_to_config(cfg, self.parent.Config)
        self.out_dir = out_dir
 

        
    def preview_scrambled(self, sbs=True):
        self.scramble()
        if sbs:
            img = helpers.merge_images(self.parent.img_orig, self.parent.im_sc)
        else:
            img = self.parent.im_sc
        scrambler.preview_img(img, 'Scrambled Image')
    
    
    def save_manual(self):
        cfg = helpers.config_to_dict(self.parent.Config, 'main')
        if self.parent.mode=='file':
            self.save_single(cfg)
        else:
            self.save_dir(cfg)
            
    def save_dir(self, cfg):
        if len(self.parent.all_out_files) < 1:
            self.parent.scramble()
        self.parent.output_dir = tkFileDialog.askdirectory(title="Select an Output Directory", initialdir=cfg['last_out_dir'])
        cfg['last_out_dir'] = self.parent.output_dir
        helpers.dict_to_config(cfg, self.parent.Config)
        for im, name in self.parent.all_out_files:
            f = os.path.join(self.parent.output_dir, name)
            cv2.imwrite(f, im)
        
            
    def save_single(self, cfg):
        try:
            img = self.parent.im_sc
        except:
            self.scramble()
            img = self.parent.im_sc
        init_file = self.parent.input_file
        init_file = os.path.basename(init_file)
        try:
            init_dir = cfg['last_out_dir']
        except:
            init_dir = ''
        out_file = tkFileDialog.asksaveasfilename(title="Select Output File Path",
                                                initialdir=init_dir,
                                                initialfile=init_file)
        if out_file:
            cv2.imwrite(out_file, img)
        
        
        

class PyScrambler(object):
    def __init__(self, master, Config):
        master.minsize(width=350, height=550)
        master.maxsize(width=1000, height=1000)
        self.all_files = []
        self.all_out_files = []
        self.img = None
        self.master = master
        self.dirmode = False
        master.title("Face Scrambler")
        cfg = helpers.config_to_dict(Config)
        self.cfg = cfg
        self.Config = Config
        self.p1 = PageLoad(self)
        self.p2 = PageOpt(self)
        self.p3 = PageScr(self)
        self.mode = 'file'
        buttonframe = tk.Frame(master)

        buttonframe.pack(side="top", fill="x", expand=False)
        self.swt_btn_text = StringVar(value="Switch to Directory Mode")
        b1 = tk.Button(buttonframe, textvariable=self.swt_btn_text, command=self.toggle_mode, state='normal')
        b1.pack()
        self.p1.pack(fill='x',expand=True)

        self.p2.pack(fill='x',expand=True)
        self.p3.pack(fill='x',expand=True)
 
        return
        
       
    
    def switch_dir(self):
        self.swt_btn_text.set('Switch to Single File Mode')
        self.p1.opn_btn_text.set('Select Directory')
        self.p3.sv_btn_text.set('Save to Directory')
    
    def switch_file(self):
        self.swt_btn_text.set('Switch to Directory Mode')
        self.p1.opn_btn_text.set('Select File')
        self.p3.sv_btn_text.set('Save to File')
        
    def toggle_mode(self):
        if self.mode == 'file':
            self.mode='dir'
        else:
            self.mode='file'
        funcstr = 'switch_' + self.mode
        func = getattr(self, funcstr)
        func()
        
    def block_buttons(self):
        self.p3.prev_button.config(state=DISABLED)
        self.p3.scramble_button.config(state=DISABLED)
        self.p3.save_button.config(state=DISABLED)
        
    def unblock_buttons(self):
        self.p3.prev_button.config(state='normal')
        self.p3.scramble_button.config(state='normal')
        self.p3.save_button.config(state='normal')
        
    def update_config(self):
        cfg = {}
        cfg['shift_up']  = self.p2.inp_offset_y.get()
        cfg['shift_right'] = self.p2.inp_offset_x.get()
        cfg['horz_size']  = self.p2.inp_horz_size.get()
        cfg['vert_size']  = self.p2.inp_vert_size.get()
        cfg['bl_sz'] = self.p2.inp_size.get()
        helpers.dict_to_config(cfg, self.Config)
        return helpers.config_to_dict(self.Config, 'main')
        
    def preview(self):
        self.update_config()
        self.scramble()
        scrambler.preview_img(self.im_sc, 'Scrambled Image')
        
    def show_orig(self):
        scrambler.preview_img(self.img, 'Original Image')
        
    def scramble(self):
        self.block_buttons()
        cfg = self.update_config()
        print(cfg)
        for key, item in list(cfg.items()):
            try:
                cfg[key] = int(item)
            except:
                pass
        if self.mode=='file':
            im = self.img.copy()
            el = scrambler.get_ellipse(im, cfg['shift_up'], cfg['shift_right'], cfg['horz_size'], cfg['vert_size'])
            self.im_sc = scrambler.scramble(im, el, (cfg['bl_sz'], cfg['bl_sz']))
        else:
            self.scramble_dir(cfg)
        self.unblock_buttons()
        
        
    def scramble_dir(self, cfg):
        for f in self.all_files:
            #print f
            try:
                img = cv2.imread(f)
                fname = os.path.basename(f)
                el = scrambler.get_ellipse(img, cfg['shift_up'], cfg['shift_right'], cfg['horz_size'], cfg['vert_size'])
                self.im_sc = scrambler.scramble(img, el, (cfg['bl_sz'], cfg['bl_sz']))
                self.all_out_files.append((self.im_sc, fname))
            except Exception as e:
                print("error while scrambling dir", e)
        
#%%
def main():
    Config, _ = helpers.get_config()
    root = Tk()
    PyScrambler(root, Config)
    root.mainloop()    
        
#%%
if __name__ == "__main__":
    main()



import sys
from face_scrambler import gui


def main(args=None):
    """The main routine."""
    if args is None:
        args = sys.argv[1:]
    gui.main()
    

if __name__ == "__main__":
    main()


from __future__ import print_function
from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
import numpy as np
import os
import configparser


def merge_images(left, right):
    h1, w1 = left.shape[:2]
    h2, w2 = right.shape[:2]    
    #create empty matrix
    vis = np.zeros((max(h1, h2), w1+w2,3), np.uint8)   
    #combine 2 images
    vis[:h1, :w1,:3] = left
    vis[:h2, w1:w1+w2,:3] = right
    return vis


def config_to_dict(Config, section='main'):
    # put dict entries into Config under section
    cfg = {}
    options = Config.options(section)
    for option in options:
        try:
            cfg[option] = Config.get(section, option)
            if cfg[option] == -1:
                print("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            cfg[option] = None
    return cfg

def dict_to_config(cfg, Config, save=True, section='main'):
    for key, val in list(cfg.items()):
        Config.set(section,key,val)
    if save:
        save_conf(Config)
    return Config

def save_conf(Config, filename='scrmbl_cfg.ini'):
    outdir = get_dir_path()
    fname = os.path.join(outdir, filename)
    with open(fname, 'w') as cfgfile:
        Config.write(cfgfile)

def get_dir_path():
    dir_path = os.path.expanduser('~')
    return dir_path

def get_config(dir_path=None, filename='scrmbl_cfg.ini'):
    dir_path = dir_path or get_dir_path()
    firstrun = False
    Config = configparser.ConfigParser()
    res = Config.read(dir_path + "/" + filename)
    if len(res) < 1:
        create_config(Config, dir_path)
        firstrun = True
    return Config, firstrun

def create_config(Config, dir_path, filename='scrmbl_cfg.ini'):
    Config.add_section('main')
    Config.set('main','shift_up',50)
    Config.set('main','shift_right', 5)
    Config.set('main','horz_size', 280)
    Config.set('main','vert_size', 375)
    Config.set('main','bl_sz', 12)
    Config.set('main','last_in_dir', dir_path)
    Config.set('main','last_out_dir', '')
    with open(dir_path + '/' + filename, 'w') as outfile:
        Config.write(outfile)